@@ -12,6 +12,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import copy
 import logging
 
 import jsonschema
@@ -32,42 +33,62 @@ def _provisioner(context):
     return metalsmith.Provisioner(session=session)
 
 
+_INSTANCE_SCHEMA = {
+    'type': 'object',
+    'properties': {
+        'capabilities': {'type': 'object'},
+        'hostname': {'type': 'string',
+                     'minLength': 2,
+                     'maxLength': 255},
+        'image': {'type': 'string'},
+        'image_checksum': {'type': 'string'},
+        'image_kernel': {'type': 'string'},
+        'image_ramdisk': {'type': 'string'},
+        'name': {'type': 'string'},
+        'nics': {'type': 'array',
+                 'items': {'type': 'object',
+                           'properties': {
+                               'network': {'type': 'string'},
+                               'port': {'type': 'string'},
+                               'fixed_ip': {'type': 'string'},
+                               'subnet': {'type': 'string'},
+                           },
+                           'additionalProperties': False}},
+        'profile': {'type': 'string'},
+        'resource_class': {'type': 'string'},
+        'root_size_gb': {'type': 'integer', 'minimum': 4},
+        'swap_size_mb': {'type': 'integer', 'minimum': 64},
+        'traits': {'type': 'array',
+                   'items': {'type': 'string'}},
+    },
+    'additionalProperties': False,
+}
+
+
 _INSTANCES_INPUT_SCHEMA = {
+    'type': 'array',
+    'items': _INSTANCE_SCHEMA
+}
+"""JSON schema of the instances list."""
+
+
+_ROLES_INPUT_SCHEMA = {
     'type': 'array',
     'items': {
         'type': 'object',
         'properties': {
-            'capabilities': {'type': 'object'},
-            'hostname': {'type': 'string',
-                         'minLength': 2,
-                         'maxLength': 255},
-            'image': {'type': 'string'},
-            'image_checksum': {'type': 'string'},
-            'image_kernel': {'type': 'string'},
-            'image_ramdisk': {'type': 'string'},
             'name': {'type': 'string'},
-            'nics': {'type': 'array',
-                     'items': {'type': 'object',
-                               'properties': {
-                                   'network': {'type': 'string'},
-                                   'port': {'type': 'string'},
-                                   'fixed_ip': {'type': 'string'},
-                                   'subnet': {'type': 'string'},
-                               },
-                               'additionalProperties': False}},
+            'count': {'type': 'integer', 'minimum': 0},
             'profile': {'type': 'string'},
-            'resource_class': {'type': 'string'},
-            'root_size_gb': {'type': 'integer', 'minimum': 4},
-            'swap_size_mb': {'type': 'integer', 'minimum': 64},
-            'traits': {'type': 'array',
-                       'items': {'type': 'string'}},
+            'hostname_format': {'type': 'string'},
+            'instances': {'type': 'array',
+                          'items': _INSTANCE_SCHEMA},
         },
         'additionalProperties': False,
-        # Host name is required, but defaults to name in _validate_instances
-        'required': ['hostname'],
+        'required': ['name'],
     }
 }
-"""JSON schema of the input for these actions."""
+"""JSON schema of the roles list."""
 
 
 class CheckExistingInstancesAction(base.TripleOAction):
@@ -113,7 +134,8 @@ class CheckExistingInstancesAction(base.TripleOAction):
                              "hostname") % (request['hostname'], instance.uuid)
                     return actions.Result(error=error)
 
-                found.append(instance.to_dict())
+                found.append(_instance_to_dict(provisioner.connection,
+                                               instance))
 
         if found:
             LOG.info('Found existing instances: %s',
@@ -241,7 +263,7 @@ class DeployNodeAction(base.TripleOAction):
 
         LOG.info('Started provisioning of %s on node %s',
                  self.instance, self.node)
-        return instance.to_dict()
+        return _instance_to_dict(provisioner.connection, instance)
 
 
 class WaitForDeploymentAction(base.TripleOAction):
@@ -269,7 +291,7 @@ class WaitForDeploymentAction(base.TripleOAction):
             )
         LOG.info('Successfully provisioned instance %s',
                  self.instance['hostname'])
-        return instance.to_dict()
+        return _instance_to_dict(provisioner.connection, instance)
 
 
 class UndeployInstanceAction(base.TripleOAction):
@@ -295,11 +317,101 @@ class UndeployInstanceAction(base.TripleOAction):
         LOG.info('Successfully unprovisioned %s', instance.hostname)
 
 
+class ExpandRolesAction(base.TripleOAction):
+    """Convert a baremetal_deployment file to list of instances."""
+
+    def __init__(self, roles, stackname='overcloud'):
+        super(ExpandRolesAction, self).__init__()
+        self.roles = roles
+        self.stackname = stackname
+
+    def run(self, context):
+        try:
+            _validate_roles(self.roles, stackname=self.stackname)
+        except Exception as exc:
+            LOG.error('Failed to validate the request. %s', exc)
+            return actions.Result(error=six.text_type(exc))
+
+        instances = []
+        parameter_defaults = {'HostnameMap': {}}
+        for role in self.roles:
+            name = role['name']
+            hostname_format = role.get('hostname_format')
+            if not hostname_format:
+                hostname_format = '%stackname%-{}-%index%'.format(
+                    'novacompute' if name == 'Compute' else name.lower())
+            # NOTE(dtantsur): our hostname format may differ from THT defaults,
+            # so override it in the resulting environment
+            parameter_defaults['%sDeployedServerHostnameFormat' % name] = (
+                hostname_format)
+
+            if 'instances' in role:
+                parameter_defaults['%sDeployedServerCount' % name] = len(
+                    role['instances'])
+                # TODO(dtantsur): ordering-dependent logic here, can we do
+                # better?
+                for index, instance in enumerate(role['instances']):
+                    # _validate_instances ensures that either of these is
+                    # not empty
+                    hostname = instance.get('hostname') or instance.get('name')
+                    gen_name = (hostname_format.replace('%index%', str(index))
+                                .replace('%stackname%', self.stackname))
+                    parameter_defaults['HostnameMap'][gen_name] = hostname
+                    instances.append(instance)
+            else:
+                count = role.get('count', 1)
+                parameter_defaults['%sDeployedServerCount' % name] = count
+                if not count:
+                    continue
+
+                for index in range(count):
+                    hostname = (hostname_format.replace('%index%', str(index))
+                                .replace('%stackname%', self.stackname))
+                    inst = {'hostname': hostname}
+                    if 'profile' in role:
+                        inst['profile'] = role['profile']
+                    instances.append(inst)
+                    parameter_defaults['HostnameMap'][hostname] = hostname
+
+        return {'instances': instances,
+                'environment': {'parameter_defaults': parameter_defaults}}
+
+
+class PopulateEnvironmentAction(base.TripleOAction):
+    """Populate the resulting environment file.
+
+    Fills in DeployedServerPortMap with the IP addresses of the nodes.
+    """
+
+    def __init__(self, environment, port_map, ctlplane_network='ctlplane'):
+        super(PopulateEnvironmentAction, self).__init__()
+        self.environment = environment
+        self.port_map = port_map
+        self.ctlplane_network = ctlplane_network
+
+    def run(self, context):
+        port_map = (self.environment.setdefault('parameter_defaults', {})
+                    .setdefault('DeployedServerPortMap', {}))
+        for hostname, nets in self.port_map.items():
+            ctlplane = nets.get(self.ctlplane_network)
+            if not ctlplane:
+                LOG.warning('No ctlplane ports information for %s', hostname)
+                continue
+
+            port_map['%s-%s' % (hostname, self.ctlplane_network)] = ctlplane
+
+        return self.environment
+
+
 def _validate_instances(instances, default_image='overcloud-full'):
     for inst in instances:
         if inst.get('name') and not inst.get('hostname'):
             inst['hostname'] = inst['name']
 
+        if not inst.get('hostname'):
+            raise ValueError('Either hostname or name is required in %s'
+                             % inst)
+
         # Set the default image so that the source validation can succeed.
         inst.setdefault('image', default_image)
 
@@ -323,6 +435,23 @@ def _validate_instances(instances, default_image='overcloud-full'):
             names.add(inst['name'])
 
 
+def _validate_roles(roles, stackname='overcloud'):
+    jsonschema.validate(roles, _ROLES_INPUT_SCHEMA)
+
+    for item in roles:
+        if 'count' in item and 'instances' in item:
+            raise ValueError("Count and instances cannot be provided together")
+
+        if 'instances' in item:
+            for index, inst in enumerate(item['instances']):
+                if 'profile' not in inst and 'profile' in item:
+                    inst['profile'] = item['profile']
+
+            # NOTE(dtantsur): make a copy since _validate_instances modifies
+            # the original, and we don't need it at this stage.
+            _validate_instances(copy.deepcopy(item['instances']))
+
+
 def _release_nodes(provisioner, nodes):
     for node in nodes:
         LOG.debug('Removing reservation from node %s', node)
@@ -339,3 +468,19 @@ def _get_source(instance):
                           kernel=instance.get('image_kernel'),
                           ramdisk=instance.get('image_ramdisk'),
                           checksum=instance.get('image_checksum'))
+
+
+def _instance_to_dict(connection, instance):
+    """Convert an instance to a dict, adding ports information."""
+    result = instance.to_dict()
+    result['port_map'] = {}
+    for nic in instance.nics():
+        for ip in nic.fixed_ips:
+            net_name = getattr(nic.network, 'name', None) or nic.network.id
+            subnet = connection.network.get_subnet(ip['subnet_id'])
+            net_info = result['port_map'].setdefault(
+                net_name, {'network': nic.network.to_dict(),
+                           'fixed_ips': [], 'subnets': []})
+            net_info['fixed_ips'].append({'ip_address': ip['ip_address']})
+            net_info['subnets'].append(subnet.to_dict())
+    return result
