@@ -21,6 +21,7 @@ import netifaces
 import os
 import re
 import requests
+from requests.adapters import HTTPAdapter
 from requests import auth as requests_auth
 import shutil
 import six
@@ -100,6 +101,38 @@ def get_undercloud_registry():
     return '%s:%s' % (addr, '8787')
 
 
+class MakeSession(object):
+    """Class method to uniformly create sessions.
+
+    Sessions created by this class will retry on errors with an exponential
+    backoff before raising an exception. Because our primary interaction is
+    with the container registries the adapter will also retry on 401 and
+    404. This is being done because registries commonly return 401 when an
+    image is not found, which is commonly a cache miss. See the adapter
+    definitions for more on retry details.
+    """
+    def __init__(self, verify=True):
+        self.session = requests.Session()
+        self.session.verify = verify
+        adapter = HTTPAdapter(
+            max_retries=8,
+            pool_connections=24,
+            pool_maxsize=24,
+            pool_block=False
+        )
+        self.session.mount('http://', adapter)
+        self.session.mount('https://', adapter)
+
+    def create(self):
+        return self.__enter__()
+
+    def __enter__(self):
+        return self.session
+
+    def __exit__(self, *args, **kwargs):
+        self.session.close()
+
+
 class ImageUploadManager(BaseImageManager):
     """Manage the uploading of image files
 
@@ -121,13 +154,15 @@ class ImageUploadManager(BaseImageManager):
         self.cleanup = cleanup
         if mirrors:
             for uploader in self.uploaders.values():
-                uploader.mirrors.update(mirrors)
+                if hasattr(uploader, 'mirrors'):
+                    uploader.mirrors.update(mirrors)
         if registry_credentials:
             self.validate_registry_credentials(registry_credentials)
             for uploader in self.uploaders.values():
                 uploader.registry_credentials = registry_credentials
 
-    def validate_registry_credentials(self, creds_data):
+    @staticmethod
+    def validate_registry_credentials(creds_data):
         if not isinstance(creds_data, dict):
             raise TypeError('Credentials data must be a dict')
         for registry, cred_entry in creds_data.items():
@@ -158,7 +193,8 @@ class ImageUploadManager(BaseImageManager):
     def get_uploader(self, uploader):
         return self.uploader(uploader)
 
-    def get_push_destination(self, item):
+    @staticmethod
+    def get_push_destination(item):
         push_destination = item.get('push_destination')
         if not push_destination:
             return get_undercloud_registry()
@@ -247,15 +283,15 @@ class BaseImageUploader(object):
     def run_modify_playbook(cls, modify_role, modify_vars,
                             source_image, target_image, append_tag,
                             container_build_tool='buildah'):
-        vars = {}
+        run_vars = {}
         if modify_vars:
-            vars.update(modify_vars)
-        vars['source_image'] = source_image
-        vars['target_image'] = target_image
-        vars['modified_append_tag'] = append_tag
-        vars['container_build_tool'] = container_build_tool
+            run_vars.update(modify_vars)
+        run_vars['source_image'] = source_image
+        run_vars['target_image'] = target_image
+        run_vars['modified_append_tag'] = append_tag
+        run_vars['container_build_tool'] = container_build_tool
         LOG.info('Playbook variables: \n%s' % yaml.safe_dump(
-            vars, default_flow_style=False))
+            run_vars, default_flow_style=False))
         playbook = [{
             'hosts': 'localhost',
             'tasks': [{
@@ -263,7 +299,7 @@ class BaseImageUploader(object):
                 'import_role': {
                     'name': modify_role
                 },
-                'vars': vars
+                'vars': run_vars
             }]
         }]
         LOG.info('Playbook: \n%s' % yaml.safe_dump(
@@ -337,14 +373,19 @@ class BaseImageUploader(object):
         wait=tenacity.wait_random_exponential(multiplier=1, max=10),
         stop=tenacity.stop_after_attempt(5)
     )
-    def authenticate(self, image_url, username=None, password=None):
+    def authenticate(self, image_url, username=None, password=None,
+                     session=None):
         netloc = image_url.netloc
         image, tag = self._image_tag_from_url(image_url)
-        self.is_insecure_registry(netloc)
+        self.is_insecure_registry(registry_host=netloc)
         url = self._build_url(image_url, path='/')
+        verify = (netloc not in self.no_verify_registries)
+        if not session:
+            session = MakeSession(verify=verify).create()
+        else:
+            session.headers.pop('Authorization', None)
+            session.verify = verify
 
-        session = requests.Session()
-        session.verify = (netloc not in self.no_verify_registries)
         r = session.get(url, timeout=30)
         LOG.debug('%s status code %s' % (url, r.status_code))
         if r.status_code == 200:
@@ -366,14 +407,123 @@ class BaseImageUploader(object):
             token_param['service'] = re.search(
                 'service="(.*?)"', www_auth).group(1)
         token_param['scope'] = 'repository:%s:pull' % image[1:]
+
         auth = None
         if username:
             auth = requests_auth.HTTPBasicAuth(username, password)
+        LOG.debug('Token parameters: params {}'.format(token_param))
         rauth = session.get(realm, params=token_param, auth=auth, timeout=30)
         rauth.raise_for_status()
         session.headers['Authorization'] = 'Bearer %s' % rauth.json()['token']
+        hash_request_id = hashlib.sha1(str(rauth.url).encode())
+        LOG.info(
+            'Session authenticated: id {}'.format(
+                hash_request_id.hexdigest()
+            )
+        )
+        setattr(session, 'reauthenticate', self.authenticate)
+        setattr(
+            session,
+            'auth_args',
+            dict(
+                image_url=image_url,
+                username=username,
+                password=password,
+                session=session
+            )
+        )
         return session
 
+    @staticmethod
+    def _get_response_text(response, encoding='utf-8', force_encoding=False):
+        """Return request response text
+
+        We need to set the encoding for the response other wise it
+        will attempt to detect the encoding which is very time consuming.
+        See https://github.com/psf/requests/issues/4235 for additional
+        context.
+
+        :param: response: requests Respoinse object
+        :param: encoding: encoding to set if not currently set
+        :param: force_encoding: set response encoding always
+        """
+
+        if force_encoding or not response.encoding:
+            response.encoding = encoding
+        return response.text
+
+    @staticmethod
+    def check_status(session, request, allow_reauth=True):
+        hash_request_id = hashlib.sha1(str(request.url).encode())
+        request_id = hash_request_id.hexdigest()
+        text = getattr(request, 'text', 'unknown')
+        reason = getattr(request, 'reason', 'unknown')
+        status_code = getattr(request, 'status_code', None)
+        headers = getattr(request, 'headers', {})
+        session_headers = getattr(session, 'headers', {})
+
+        if status_code >= 300:
+            LOG.info(
+                'Non-2xx: id {}, status {}, reason {}, text {}'.format(
+                    request_id,
+                    status_code,
+                    reason,
+                    text
+                )
+            )
+
+        if status_code == 401:
+            LOG.warning(
+                'Failure: id {}, status {}, reason {} text {}'.format(
+                    request_id,
+                    status_code,
+                    reason,
+                    text
+                )
+            )
+            LOG.debug(
+                'Request headers after 401: id {}, headers {}'.format(
+                    request_id,
+                    headers
+                )
+            )
+            LOG.debug(
+                'Session headers after 401: id {}, headers {}'.format(
+                    request_id,
+                    session_headers
+                )
+            )
+
+            www_auth = headers.get(
+                'www-authenticate',
+                headers.get(
+                    'Www-Authenticate'
+                )
+            )
+            if www_auth:
+                error = None
+                if 'error=' in www_auth:
+                    error = re.search('error="(.*?)"', www_auth).group(1)
+                    LOG.warning(
+                        'Error detected in auth headers: error {}'.format(
+                            error
+                        )
+                    )
+                if error == 'invalid_token' and allow_reauth:
+                    if hasattr(session, 'reauthenticate'):
+                        reauth = int(session.headers.get('_TripleOReAuth', 0))
+                        reauth += 1
+                        session.headers['_TripleOReAuth'] = str(reauth)
+                        LOG.warning(
+                            'Re-authenticating: id {}, count {}'.format(
+                                request_id,
+                                reauth
+                            )
+                        )
+                        session.reauthenticate(**session.auth_args)
+
+        request.raise_for_status()
+
     @classmethod
     def _build_url(cls, url, path):
         netloc = url.netloc
@@ -381,10 +531,10 @@ class BaseImageUploader(object):
             mirror = cls.mirrors[netloc]
             return '%sv2%s' % (mirror, path)
         else:
-            if netloc in cls.insecure_registries:
-                scheme = 'http'
-            else:
+            if not cls.is_insecure_registry(registry_host=netloc):
                 scheme = 'https'
+            else:
+                scheme = 'http'
             if netloc == 'docker.io':
                 netloc = 'registry-1.docker.io'
             return '%s://%s/v2%s' % (scheme, netloc, path)
@@ -426,15 +576,15 @@ class BaseImageUploader(object):
         tags_f = p.submit(session.get, tags_url, timeout=30)
 
         manifest_r = manifest_f.result()
-        tags_r = tags_f.result()
-
         if manifest_r.status_code in (403, 404):
             raise ImageNotFoundException('Not found image: %s' %
                                          image_url.geturl())
-        manifest_r.raise_for_status()
-        tags_r.raise_for_status()
+        cls.check_status(session=session, request=manifest_r)
 
-        manifest_str = manifest_r.text
+        tags_r = tags_f.result()
+        cls.check_status(session=session, request=tags_r)
+
+        manifest_str = cls._get_response_text(manifest_r)
 
         if 'Docker-Content-Digest' in manifest_r.headers:
             digest = manifest_r.headers['Docker-Content-Digest']
@@ -462,7 +612,7 @@ class BaseImageUploader(object):
             config_f = p.submit(
                 session.get, config_url, headers=config_headers, timeout=30)
             config_r = config_f.result()
-            config_r.raise_for_status()
+            cls.check_status(session=session, request=config_r)
             config = config_r.json()
 
         tags = tags_r.json()['tags']
@@ -489,7 +639,7 @@ class BaseImageUploader(object):
         }
 
     def list(self, registry, session=None):
-        self.is_insecure_registry(registry)
+        self.is_insecure_registry(registry_host=registry)
         url = self._image_to_url(registry)
         catalog_url = self._build_url(
             url, CALL_CATALOG
@@ -502,7 +652,7 @@ class BaseImageUploader(object):
         else:
             raise ImageUploaderException(
                 'Image registry made invalid response: %s' %
-                (catalog_resp.status_code)
+                catalog_resp.status_code
             )
 
         tags_get_args = []
@@ -566,7 +716,10 @@ class BaseImageUploader(object):
                                    fallback_tag=None):
         labels = i.get('Labels', {})
 
-        label_keys = ', '.join(labels.keys())
+        if hasattr(labels, 'keys'):
+            label_keys = ', '.join(labels.keys())
+        else:
+            label_keys = ""
 
         if not tag_from_label:
             raise ImageUploaderException(
@@ -587,7 +740,9 @@ class BaseImageUploader(object):
                         (image, e, label_keys)
                     )
         else:
-            tag_label = labels.get(tag_from_label)
+            tag_label = None
+            if isinstance(labels, dict):
+                tag_label = labels.get(tag_from_label)
             if tag_label is None:
                 if fallback_tag:
                     tag_label = fallback_tag
@@ -611,7 +766,7 @@ class BaseImageUploader(object):
 
         # prime self.insecure_registries by testing every image
         for url in image_urls:
-            self.is_insecure_registry(url)
+            self.is_insecure_registry(registry_host=url)
 
         discover_args = []
         for image in images:
@@ -627,7 +782,7 @@ class BaseImageUploader(object):
     def discover_image_tag(self, image, tag_from_label=None,
                            fallback_tag=None, username=None, password=None):
         image_url = self._image_to_url(image)
-        self.is_insecure_registry(image_url.netloc)
+        self.is_insecure_registry(registry_host=image_url.netloc)
         session = self.authenticate(
             image_url, username=username, password=password)
 
@@ -640,7 +795,7 @@ class BaseImageUploader(object):
         images_with_labels = []
         for image in images:
             url = self._image_to_url(image)
-            self.is_insecure_registry(url.netloc)
+            self.is_insecure_registry(registry_host=url.netloc)
             session = self.authenticate(
                 url, username=username, password=password)
             image_labels = self._image_labels(
@@ -654,39 +809,50 @@ class BaseImageUploader(object):
         # prime insecure_registries
         if task.pull_source:
             self.is_insecure_registry(
-                self._image_to_url(task.pull_source).netloc)
+                registry_host=self._image_to_url(task.pull_source).netloc
+            )
         else:
             self.is_insecure_registry(
-                self._image_to_url(task.image_name).netloc)
+                registry_host=self._image_to_url(task.image_name).netloc
+            )
         self.is_insecure_registry(
-            self._image_to_url(task.push_destination).netloc)
+            registry_host=self._image_to_url(task.push_destination).netloc
+        )
         self.upload_tasks.append((self, task))
 
-    def is_insecure_registry(self, registry_host):
-        if registry_host in self.secure_registries:
+    @classmethod
+    def is_insecure_registry(cls, registry_host):
+        if registry_host in cls.secure_registries:
             return False
-        if registry_host in self.insecure_registries:
+        if (registry_host in cls.insecure_registries or
+                registry_host in cls.no_verify_registries):
             return True
-        try:
-            requests.get('https://%s/v2' % registry_host, timeout=30)
-        except requests.exceptions.SSLError:
-            # Might be just a TLS certificate validation issue
-            # Just retry without the verification
+        with requests.Session() as s:
             try:
-                requests.get('https://%s/v2' % registry_host, timeout=30,
-                             verify=False)
-                self.no_verify_registries.add(registry_host)
-                return False
+                s.get('https://%s/v2' % registry_host, timeout=30)
             except requests.exceptions.SSLError:
-                # So nope, it's really not a certificate verification issue
-                self.insecure_registries.add(registry_host)
-                return True
-        except Exception:
-            # for any other error assume it is a secure registry, because:
-            # - it is secure registry
-            # - the host is not accessible
-            pass
-        self.secure_registries.add(registry_host)
+                # Might be just a TLS certificate validation issue
+                # Just retry without the verification
+                try:
+                    s.get('https://%s/v2' % registry_host, timeout=30,
+                          verify=False)
+                    cls.no_verify_registries.add(registry_host)
+                    # Techinically these type of registries are insecure when
+                    # the container engine tries to do a pull. The python
+                    # uploader ignores the certificate problem, but they are
+                    # still inscure so we return True here while we'll still
+                    # use https when we access the registry. LP#1833751
+                    return True
+                except requests.exceptions.SSLError:
+                    # So nope, it's really not a certificate verification issue
+                    cls.insecure_registries.add(registry_host)
+                    return True
+            except Exception:
+                # for any other error assume it is a secure registry, because:
+                # - it is secure registry
+                # - the host is not accessible
+                pass
+        cls.secure_registries.add(registry_host)
         return False
 
     @classmethod
@@ -724,7 +890,7 @@ class BaseImageUploader(object):
                     'from': existing_name
                 }
                 r = session.post(url, data=data, timeout=30)
-                r.raise_for_status()
+                cls.check_status(session=session, request=r)
                 LOG.debug('%s %s' % (r.status_code, r.reason))
 
 
@@ -845,9 +1011,8 @@ class SkopeoImageUploader(BaseImageUploader):
                                          (' '.join(cmd), err))
         return out
 
-    @classmethod
-    def _delete(cls, image_url, session=None):
-        insecure = cls.is_insecure_registry(image_url.netloc)
+    def _delete(self, image_url, session=None):
+        insecure = self.is_insecure_registry(registry_host=image_url.netloc)
         image = image_url.geturl()
         LOG.info('Deleting %s' % image)
         cmd = ['skopeo', 'delete']
@@ -885,14 +1050,11 @@ class SkopeoImageUploader(BaseImageUploader):
 
         # Pull a single image first, to avoid duplicate pulls of the
         # same base layers
-        uploader, first_task = self.upload_tasks.pop()
-        result = uploader.upload_image(first_task)
-        local_images.extend(result)
+        local_images.extend(upload_task(args=self.upload_tasks.pop()))
 
         # workers will be half the CPU count, to a minimum of 2
-        workers = max(2, processutils.get_worker_count() // 2)
+        workers = max(2, (processutils.get_worker_count() - 1))
         p = futures.ThreadPoolExecutor(max_workers=workers)
-
         for result in p.map(upload_task, self.upload_tasks):
             local_images.extend(result)
         LOG.info('result %s' % local_images)
@@ -929,6 +1091,7 @@ class PythonImageUploader(BaseImageUploader):
                     t.target_image, target_session):
                 LOG.warning('Skipping upload for modified image %s' %
                             t.target_image)
+                target_session.close()
                 return []
             copy_target_url = t.target_image_source_tag_url
         else:
@@ -970,7 +1133,11 @@ class PythonImageUploader(BaseImageUploader):
         if not t.modify_role:
             LOG.warning('Completed upload for image %s' % t.image_name)
         else:
-            # Copy ummodified from target to local
+            LOG.info(
+                'Copy ummodified imagename: "{}" from target to local'.format(
+                    t.image_name
+                )
+            )
             self._copy_registry_to_local(t.target_image_source_tag_url)
 
             if t.cleanup in (CLEANUP_FULL, CLEANUP_PARTIAL):
@@ -1004,6 +1171,9 @@ class PythonImageUploader(BaseImageUploader):
                         t.image_name)
         for layer in source_layers:
             self.image_layers.setdefault(layer, t.target_image_url)
+
+        target_session.close()
+        source_session.close()
         return to_cleanup
 
     @classmethod
@@ -1022,7 +1192,7 @@ class PythonImageUploader(BaseImageUploader):
             return False
 
         # detect if the registry is push-capable by requesting an upload URL.
-        image, tag = cls._image_tag_from_url(image_url)
+        image, _ = cls._image_tag_from_url(image_url)
         upload_req_url = cls._build_url(
             image_url,
             path=CALL_UPLOAD % {'image': image})
@@ -1030,7 +1200,7 @@ class PythonImageUploader(BaseImageUploader):
         if r.status_code in (501, 403, 404, 405):
             cls.export_registries.add(image_url.netloc)
             return True
-        r.raise_for_status()
+        cls.check_status(session=session, request=r)
         cls.push_registries.add(image_url.netloc)
         return False
 
@@ -1056,8 +1226,8 @@ class PythonImageUploader(BaseImageUploader):
         r = session.get(url, headers=manifest_headers, timeout=30)
         if r.status_code in (403, 404):
             raise ImageNotFoundException('Not found image: %s' % url)
-        r.raise_for_status()
-        return r.text
+        cls.check_status(session=session, request=r)
+        return cls._get_response_text(r)
 
     @classmethod
     @tenacity.retry(  # Retry up to 5 times with jittered exponential backoff
@@ -1077,7 +1247,7 @@ class PythonImageUploader(BaseImageUploader):
             image_url,
             path=CALL_UPLOAD % {'image': image})
         r = session.post(upload_req_url, timeout=30)
-        r.raise_for_status()
+        cls.check_status(session=session, request=r)
         return r.headers['Location']
 
     @classmethod
@@ -1103,7 +1273,9 @@ class PythonImageUploader(BaseImageUploader):
         chunk_size = 2 ** 20
         with session.get(
                 source_blob_url, stream=True, timeout=30) as blob_req:
-            blob_req.raise_for_status()
+            # TODO(aschultz): unsure if necessary or if only when using .text
+            blob_req.encoding = 'utf-8'
+            cls.check_status(session=session, request=blob_req)
             for data in blob_req.iter_content(chunk_size):
                 if not data:
                     break
@@ -1256,7 +1428,7 @@ class PythonImageUploader(BaseImageUploader):
                     'Content-Type': 'application/octet-stream'
                 }
             )
-            r.raise_for_status(),
+            cls.check_status(session=target_session, request=r)
 
         # Upload the manifest
         image, tag = cls._image_tag_from_url(target_url)
@@ -1279,9 +1451,9 @@ class PythonImageUploader(BaseImageUploader):
             }
         )
         if r.status_code == 400:
-            LOG.error(r.text)
+            LOG.error(cls._get_response_text(r))
             raise ImageUploaderException('Pushing manifest failed')
-        r.raise_for_status()
+        cls.check_status(session=target_session, request=r)
 
     @classmethod
     @tenacity.retry(  # Retry up to 5 times with jittered exponential backoff
@@ -1292,25 +1464,34 @@ class PythonImageUploader(BaseImageUploader):
     def _copy_registry_to_local(cls, source_url):
         cls._assert_scheme(source_url, 'docker')
         pull_source = source_url.netloc + source_url.path
-        LOG.info('Pulling %s' % pull_source)
-        cmd = ['buildah', 'pull']
+        cmd = ['buildah', '--debug', 'pull']
 
         if source_url.netloc in [cls.insecure_registries,
                                  cls.no_verify_registries]:
             cmd.append('--tls-verify=false')
 
         cmd.append(pull_source)
+        LOG.info('Pulling %s' % pull_source)
         LOG.info('Running %s' % ' '.join(cmd))
-        env = os.environ.copy()
-        process = subprocess.Popen(cmd, env=env, stdout=subprocess.PIPE,
-                                   universal_newlines=True)
-
+        process = subprocess.Popen(
+            cmd,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+            universal_newlines=True,
+            close_fds=True
+        )
         out, err = process.communicate()
-        LOG.info(out)
         if process.returncode != 0:
-            LOG.error('Error pulling image:\n%s\n%s' %
-                      (' '.join(cmd), err))
-            raise ImageUploaderException('Pulling image failed')
+            error_msg = (
+                'Pulling image failed: cmd "{}", stdout "{}",'
+                ' stderr "{}"'.format(
+                    ' '.join(cmd),
+                    out,
+                    err
+                )
+            )
+            LOG.error(error_msg)
+            raise ImageUploaderException(error_msg)
         return out
 
     @classmethod
@@ -1444,7 +1625,7 @@ class PythonImageUploader(BaseImageUploader):
                     'Content-Type': 'application/octet-stream'
                 }
             )
-            upload_resp.raise_for_status()
+            cls.check_status(session=session, request=upload_resp)
             length += chunk_length
 
         layer_digest = 'sha256:%s' % calc_digest.hexdigest()
@@ -1458,7 +1639,7 @@ class PythonImageUploader(BaseImageUploader):
                 'digest': layer_digest
             },
         )
-        upload_resp.raise_for_status()
+        cls.check_status(session=session, request=upload_resp)
         layer['digest'] = layer_digest
         layer['size'] = length
         return layer_digest
@@ -1570,7 +1751,7 @@ class PythonImageUploader(BaseImageUploader):
         config = json.loads(config_str)
 
         layers = [l['digest'] for l in manifest['layers']]
-        i, tag = cls._image_tag_from_url(image_url)
+        i, _ = cls._image_tag_from_url(image_url)
         digest = image['digest']
         created = image['created']
         labels = config['config']['Labels']
@@ -1634,14 +1815,11 @@ class PythonImageUploader(BaseImageUploader):
 
         # Pull a single image first, to avoid duplicate pulls of the
         # same base layers
-        uploader, first_task = self.upload_tasks.pop()
-        result = uploader.upload_image(first_task)
-        local_images.extend(result)
+        local_images.extend(upload_task(args=self.upload_tasks.pop()))
 
-        # workers will be half the CPU count, to a minimum of 2
-        workers = max(2, processutils.get_worker_count() // 2)
+        # workers will the CPU count minus 1, with a minimum of 2
+        workers = max(2, (processutils.get_worker_count() - 1))
         p = futures.ThreadPoolExecutor(max_workers=workers)
-
         for result in p.map(upload_task, self.upload_tasks):
             local_images.extend(result)
         LOG.info('result %s' % local_images)
@@ -1687,7 +1865,8 @@ class UploadTask(object):
         self.source_image_url = image_to_url(self.source_image)
         self.target_image_url = image_to_url(self.target_image)
         self.target_image_source_tag_url = image_to_url(
-            self.target_image_source_tag)
+            self.target_image_source_tag
+        )
 
 
 def upload_task(args):
@@ -1702,6 +1881,7 @@ def discover_tag_from_inspect(args):
     session = self.authenticate(
         image_url, username=username, password=password)
     i = self._inspect(image_url, session=session)
+    session.close()
     if ':' in image_url.path:
         # break out the tag from the url to be the fallback tag
         path = image.rpartition(':')
