@@ -53,6 +53,8 @@ class TestImageUploadManager(base.TestCase):
         files.append('testfile')
         self.filelist = files
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader._fetch_manifest')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -75,7 +77,8 @@ class TestImageUploadManager(base.TestCase):
                 'get_undercloud_registry', return_value='192.0.2.0:8787')
     def test_file_parsing(self, mock_gur, mockioctl, mockpath,
                           mock_images_match, mock_is_insecure, mock_inspect,
-                          mock_auth, mock_copy, mock_manifest):
+                          mock_auth, mock_copy, mock_manifest,
+                          check_status):
 
         mock_manifest.return_value = '{"layers": []}'
         mock_inspect.return_value = {}
@@ -227,45 +230,56 @@ class TestBaseImageUploader(base.TestCase):
         self.uploader._inspect.retry.sleep = mock.Mock()
         self.requests = self.useFixture(rm_fixture.Fixture())
 
-    def test_is_insecure_registry_known(self):
+    @mock.patch.object(requests.Session, 'get', return_value=True)
+    def test_is_insecure_registry_known(self, mock_session):
         self.assertFalse(
             self.uploader.is_insecure_registry('docker.io'))
 
-    def test_is_insecure_registry_secure(self):
+    @mock.patch.object(requests.Session, 'get', return_value=True)
+    def test_is_insecure_registry_secure(self, mock_session):
         self.assertFalse(
             self.uploader.is_insecure_registry('192.0.2.0:8787'))
         self.assertFalse(
             self.uploader.is_insecure_registry('192.0.2.0:8787'))
-        self.assertEqual(
-            'https://192.0.2.0:8787/v2',
-            self.requests.request_history[0].url
-        )
+        calls = [mock.call('https://192.0.2.0:8787/v2', timeout=30)]
+        mock_session.assert_has_calls(calls)
+        self.assertEqual(mock_session.call_count, 1)
 
-    def test_is_insecure_registry_timeout(self):
-        self.requests.get(
-            'https://192.0.2.0:8787/',
-            exc=requests.exceptions.ReadTimeout('ouch'))
-        self.assertFalse(
-            self.uploader.is_insecure_registry('192.0.2.0:8787'))
-        self.assertFalse(
-            self.uploader.is_insecure_registry('192.0.2.0:8787'))
-        self.assertEqual(
-            'https://192.0.2.0:8787/v2',
-            self.requests.request_history[0].url
-        )
+    @mock.patch.object(requests.Session, 'get',
+                       side_effect=[requests.exceptions.SSLError('err'), True])
+    def test_is_insecure_registry_bad_cert(self, mock_session):
+        self.assertTrue(
+            self.uploader.is_insecure_registry('bcert:8787'))
+        self.assertTrue(
+            self.uploader.is_insecure_registry('bcert:8787'))
+        calls = [mock.call('https://bcert:8787/v2', timeout=30),
+                 mock.call('https://bcert:8787/v2', timeout=30, verify=False)]
+        mock_session.assert_has_calls(calls)
+        self.assertEqual(mock_session.call_count, 2)
 
-    def test_is_insecure_registry_insecure(self):
-        self.requests.get(
-            'https://192.0.2.0:8787/v2',
-            exc=requests.exceptions.SSLError('ouch'))
+    @mock.patch.object(requests.Session, 'get',
+                       side_effect=requests.exceptions.ReadTimeout('ouch'))
+    def test_is_insecure_registry_timeout(self, mock_session):
+        self.assertFalse(
+            self.uploader.is_insecure_registry('192.0.2.0:8787'))
+        self.assertFalse(
+            self.uploader.is_insecure_registry('192.0.2.0:8787'))
+        calls = [mock.call('https://192.0.2.0:8787/v2', timeout=30)]
+        mock_session.assert_has_calls(calls)
+        self.assertEqual(mock_session.call_count, 1)
+
+    @mock.patch.object(requests.Session, 'get',
+                       side_effect=requests.exceptions.SSLError('ouch'))
+    def test_is_insecure_registry_insecure(self, mock_session):
         self.assertTrue(
             self.uploader.is_insecure_registry('192.0.2.0:8787'))
         self.assertTrue(
             self.uploader.is_insecure_registry('192.0.2.0:8787'))
-        self.assertEqual(
-            'https://192.0.2.0:8787/v2',
-            self.requests.request_history[0].url
-        )
+        calls = [mock.call('https://192.0.2.0:8787/v2', timeout=30),
+                 mock.call('https://192.0.2.0:8787/v2', timeout=30,
+                           verify=False)]
+        mock_session.assert_has_calls(calls)
+        self.assertEqual(mock_session.call_count, 2)
 
     @mock.patch('tripleo_common.image.image_uploader.'
                 'BaseImageUploader.authenticate')
@@ -516,7 +530,7 @@ class TestBaseImageUploader(base.TestCase):
         insecure_reg.add('registry-1.docker.io')
         secure_reg.add('192.0.2.1:8787')
         self.assertEqual(
-            'http://registry-1.docker.io/v2/t/nova-api/manifests/latest',
+            'https://registry-1.docker.io/v2/t/nova-api/manifests/latest',
             build(url2, '/t/nova-api/manifests/latest')
         )
         self.assertEqual(
@@ -859,6 +873,8 @@ class TestSkopeoImageUploader(base.TestCase):
         self.uploader._copy.retry.sleep = mock.Mock()
         self.uploader._inspect.retry.sleep = mock.Mock()
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('os.environ')
     @mock.patch('subprocess.Popen')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -866,7 +882,7 @@ class TestSkopeoImageUploader(base.TestCase):
     @mock.patch('tripleo_common.image.image_uploader.'
                 'BaseImageUploader.authenticate')
     def test_upload_image(self, mock_auth, mock_inspect,
-                          mock_popen, mock_environ):
+                          mock_popen, mock_environ, check_status):
         mock_process = mock.Mock()
         mock_process.communicate.return_value = ('copy complete', '')
         mock_process.returncode = 0
@@ -1134,10 +1150,11 @@ class TestPythonImageUploader(base.TestCase):
         u._copy_layer_local_to_registry.retry.sleep = mock.Mock()
         u._copy_layer_registry_to_registry.retry.sleep = mock.Mock()
         u._copy_registry_to_registry.retry.sleep = mock.Mock()
-        u._copy_registry_to_local.retry.sleep = mock.Mock()
         u._copy_local_to_registry.retry.sleep = mock.Mock()
         self.requests = self.useFixture(rm_fixture.Fixture())
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.authenticate')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1148,7 +1165,7 @@ class TestPythonImageUploader(base.TestCase):
                 'PythonImageUploader._copy_registry_to_registry')
     def test_upload_image(
             self, _copy_registry_to_registry, _cross_repo_mount,
-            _fetch_manifest, authenticate):
+            _fetch_manifest, authenticate, check_status):
 
         target_session = mock.Mock()
         source_session = mock.Mock()
@@ -1226,6 +1243,8 @@ class TestPythonImageUploader(base.TestCase):
             target_session=target_session
         )
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.authenticate')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1236,7 +1255,7 @@ class TestPythonImageUploader(base.TestCase):
                 'PythonImageUploader._copy_registry_to_registry')
     def test_authenticate_upload_image(
             self, _copy_registry_to_registry, _cross_repo_mount,
-            _fetch_manifest, authenticate):
+            _fetch_manifest, authenticate, check_status):
 
         self.uploader.registry_credentials = {
             'docker.io': {'my_username': 'my_password'},
@@ -1297,6 +1316,8 @@ class TestPythonImageUploader(base.TestCase):
             ),
         ])
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.authenticate')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1307,7 +1328,7 @@ class TestPythonImageUploader(base.TestCase):
                 'PythonImageUploader._copy_registry_to_registry')
     def test_insecure_registry(
             self, _copy_registry_to_registry, _cross_repo_mount,
-            _fetch_manifest, authenticate):
+            _fetch_manifest, authenticate, check_status):
         target_session = mock.Mock()
         source_session = mock.Mock()
         authenticate.side_effect = [
@@ -1363,6 +1384,8 @@ class TestPythonImageUploader(base.TestCase):
             ),
         ])
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.authenticate')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1373,7 +1396,7 @@ class TestPythonImageUploader(base.TestCase):
                 'PythonImageUploader._copy_registry_to_registry')
     def test_upload_image_v1_manifest(
             self, _copy_registry_to_registry, _cross_repo_mount,
-            _fetch_manifest, authenticate):
+            _fetch_manifest, authenticate, check_status):
 
         target_session = mock.Mock()
         source_session = mock.Mock()
@@ -1449,6 +1472,8 @@ class TestPythonImageUploader(base.TestCase):
             target_session=target_session
         )
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.authenticate')
     @mock.patch('tripleo_common.image.image_uploader.'
@@ -1468,7 +1493,8 @@ class TestPythonImageUploader(base.TestCase):
     def test_upload_image_modify(
             self, _copy_local_to_registry, run_modify_playbook,
             _copy_registry_to_local, _copy_registry_to_registry,
-            _cross_repo_mount, _fetch_manifest, _image_exists, authenticate):
+            _cross_repo_mount, _fetch_manifest, _image_exists, authenticate,
+            check_status):
 
         _image_exists.return_value = False
         target_session = mock.Mock()
@@ -1588,7 +1614,9 @@ class TestPythonImageUploader(base.TestCase):
             session=target_session
         )
 
-    def test_fetch_manifest(self):
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
+    def test_fetch_manifest(self, check_status):
         url = urlparse('docker://docker.io/t/nova-api:tripleo-current')
         manifest = '{"layers": []}'
         session = mock.Mock()
@@ -1608,7 +1636,9 @@ class TestPythonImageUploader(base.TestCase):
             }
         )
 
-    def test_upload_url(self):
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
+    def test_upload_url(self, check_status):
         # test with previous request
         previous_request = mock.Mock()
         previous_request.headers = {
@@ -1729,12 +1759,15 @@ class TestPythonImageUploader(base.TestCase):
             'docker'
         )
 
+    @mock.patch('tripleo_common.image.image_uploader.'
+                'BaseImageUploader.check_status')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader._upload_url')
     @mock.patch('tripleo_common.image.image_uploader.'
                 'PythonImageUploader.'
                 '_copy_layer_registry_to_registry')
-    def test_copy_registry_to_registry(self, _copy_layer, _upload_url):
+    def test_copy_registry_to_registry(self, _copy_layer, _upload_url,
+                                       check_status):
         source_url = urlparse('docker://docker.io/t/nova-api:latest')
         target_url = urlparse('docker://192.168.2.1:5000/t/nova-api:latest')
         _upload_url.return_value = 'https://192.168.2.1:5000/v2/upload'
@@ -1795,7 +1828,6 @@ class TestPythonImageUploader(base.TestCase):
                 params={'digest': 'sha256:1234'},
                 timeout=30
             ),
-            mock.call().raise_for_status(),
             mock.call(
                 'https://192.168.2.1:5000/v2/t/nova-api/manifests/latest',
                 data=mock.ANY,
@@ -1805,7 +1837,6 @@ class TestPythonImageUploader(base.TestCase):
                 },
                 timeout=30
             ),
-            mock.call().raise_for_status(),
         ])
         put_manifest = json.loads(
             target_session.put.call_args[1]['data'].decode('utf-8')
