@@ -39,7 +39,8 @@ MOCK_ENABLED_SERVICES = {
     "Compute": [
         "nova_compute",
         "kernel",
-        "tripleo_packages"
+        "tripleo_packages",
+        "ceph_client"
     ],
     "CephStorage": [
         "kernel",
@@ -60,7 +61,7 @@ class TestInventory(base.TestCase):
             {'output_key': 'EnabledServices',
              'output_value': {
                  'Controller': ['sa', 'sb'],
-                 'Compute': ['sd', 'se'],
+                 'Compute': ['sd', 'se', 'ceph_client'],
                  'CustomRole': ['sg', 'sh']}},
             {'output_key': 'KeystoneURL',
              'output_value': 'xyz://keystone'},
@@ -138,6 +139,7 @@ class TestInventory(base.TestCase):
             'keystone': ['Controller'],
             'nova_compute': ['Compute'],
             'cinder_volume': ['BlockStorage'],
+            'ceph_client': ['Compute'],
         }
         self.assertDictEqual(services, expected)
 
@@ -280,6 +282,24 @@ class TestInventory(base.TestCase):
                     'container_cli': 'podman',
                     'ctlplane_vip': 'x.x.x.4',
                     'redis_vip': 'x.x.x.6'}},
+            'sa': {'children': ['Controller'],
+                   'vars': {'ansible_ssh_user': 'my-custom-admin',
+                            'ansible_python_interpreter': 'foo'}},
+            'sb': {'children': ['Controller'],
+                   'vars': {'ansible_ssh_user': 'my-custom-admin',
+                            'ansible_python_interpreter': 'foo'}},
+            'sd': {'children': ['Compute'],
+                   'vars': {'ansible_ssh_user': 'my-custom-admin',
+                            'ansible_python_interpreter': 'foo'}},
+            'se': {'children': ['Compute'],
+                   'vars': {'ansible_ssh_user': 'my-custom-admin',
+                            'ansible_python_interpreter': 'foo'}},
+            'sg': {'children': ['CustomRole'],
+                   'vars': {'ansible_ssh_user': 'my-custom-admin',
+                            'ansible_python_interpreter': 'foo'}},
+            'sh': {'children': ['CustomRole'],
+                   'vars': {'ansible_ssh_user': 'my-custom-admin',
+                            'ansible_python_interpreter': 'foo'}},
             'Undercloud': {
                 'hosts': ['undercloud'],
                 'vars': {'ansible_connection': 'ssh',
@@ -380,6 +400,10 @@ class TestInventory(base.TestCase):
                    'vars': {'ansible_ssh_user': 'heat-admin'}},
             'se': {'children': {'Compute': {}},
                    'vars': {'ansible_ssh_user': 'heat-admin'}},
+            'ceph_client': {'children': {'Compute': {}},
+                            'vars': {'ansible_ssh_user': 'heat-admin'}},
+            'clients': {'children': {'Compute': {}},
+                        'vars': {'ansible_ssh_user': 'heat-admin'}},
             'sg': {'children': {'CustomRole': {}},
                    'vars': {'ansible_ssh_user': 'heat-admin'}},
             'sh': {'children': {'CustomRole': {}},
